# -*- coding: utf-8 -*-

import os
import traceback
from functools import wraps

from lxml import etree as etree_lxml
from upload.converter.base_converter import (Converter)
from upload.converter.smagPlugins.xml.converter_ukm_xml import ConverterUkm

errorObjectsCount = 0


def check_shop_code(func):
    """
    Сравнение кода магазина, указанного в справочнике, с кодом магазина у ноды
    Если коды не совпадают - справочник не загружаем
    """

    @wraps(func)
    def wrapper(self, *args):
        self.log.debug("Проверка соответствия кода магазина")
        tree = args[0]
        root = tree.getroot()
        store_id = root.attrib["storeId"]
        if store_id == self.shopcode:
            return func(self, tree)
        else:
            self.log.warn("Код магазина в справочнике не совпадает с текущим, справочник не будет загружен!")
            return []

    return wrapper


def get_aif_objects(func):
    """
    Получение aif-объектов для загрузки
    """

    @wraps(func)
    def wrapper(self, *args):
        tree = args[0]
        objs = func(self, tree)
        root = tree.getroot()
        return self.formatter.get_command(root.tag, objs)

    return wrapper


class SmagUkmXMLConverter(Converter):
    tnvds = dict()

    def __init__(self, node):
        super(SmagUkmXMLConverter, self).__init__(node)
        self.work_dir = None
        self.errorList = list()
        self.filePosition = 0
        self.currentFileSize = 0
        self.currentFile = None
        self.objects = []
        self.formatter = ConverterUkm()
        self.parser = {
            "updateItems": lambda xml_tree: self.parse_invent(xml_tree),
            "storePrices": lambda xml_tree: self.parse_price(xml_tree),
            "storePricesAdd": lambda xml_tree: self.parse_tprice(xml_tree),
            "taxes": lambda xml_tree: self.parse_tax(xml_tree),
            "classif": lambda xml_tree: self.parse_group(xml_tree),
            "kit": lambda xml_tree: self.parse_kit(xml_tree)
        }
        self.formatter.tnvdMapper = node.currentConverter.tnvdMapper
        self.formatter.measureMapper = node.currentConverter.measureMapper

        self.tnvd_map = dict()

    def parse(self, data):
        """
        @override

        Метод парсинга всех файлов
        data - список файлов
        Возвращает результат вставки (aif-объектов, список ошибок)
        """
        if not self.inserter.start():
            self.log.error(u'Не удалось запустить Inserter')
            return False, self.errorList
        try:
            for filePath in data:
                self.parseSingleFile(filePath)
        except Exception as e:
            print(e)
            self.log.error(str(e) + (u'(позиция в файле %s)' % self.filePosition))
            self.log.error(traceback.format_exc())
            self.errorList.append(e)
            return False, self.errorList
        result = self.inserter.stop()
        global errorObjectsCount
        errorObjectsCount = self.inserter.invalidObjectsCount()
        return result, self.errorList

    def parseSingleFile(self, file_path):
        """
        Парсинг данных из xml-файла
        filePath - путь до файла
        """
        tree = etree_lxml.parse(file_path)
        root = tree.getroot()
        if root.tag != 'CodeTNVED' and root.tag in self.parser:
            objs = self.parser[root.tag](tree)
            self.load_objects(list(objs), len(objs))
        elif root.tag != 'CodeTNVED':
            self.log.warn(u"Работа с тегом %s не поддержана" % root.tag)

    def parse_invent(self, tree):
        """
        получение товаров из xml-файла
        :param tree: xml
        :return:
        """

        # считываем коды тнвд из файла, чтобы потом искать по ним tmcType
        tnvds = self.get_tnvd()

        alcocodes = list()
        tmcs = list()
        for action, elem in etree_lxml.iterwalk(tree, tag="item"):
            tmc = {x.tag: x.text for x in list(elem.getchildren())}
            # считываем список ШК товара из xml
            barcodes_xml = elem.findall('barcode')
            barcodes = list()
            if len(barcodes_xml) > 0:
                for bcode in barcodes_xml:
                    barcodes.append({x.tag: x.text for x in bcode.getchildren()})
            tmc['barcode'] = barcodes
            # считываем список алкокодов товара из xml
            if elem.find('ProductAlcCodes') is not None:
                for alc_code in elem.find('ProductAlcCodes').getchildren():
                    # для алкокодов дополнительно добавляем код товара, чтобы заполнить таблицу tmcalcocode
                    alcocodes.append({alc_code.tag: alc_code.text, "article": tmc['article']})
            tmcs.append(tmc)
        # выполняется маппинг объектов в артикс-сущности
        # получаем команды на загрузку в таблицы tmc, barcode, alcocode, tmcalcocode
        add_invent_commands = self.formatter.get_command(tree.getroot().tag, tmcs,
                                                         kwargs={'tnvds': tnvds})  # type: list
        if len(alcocodes) > 0:
            add_tmc_alcocodes = self.formatter.get_command('alcocode', alcocodes)  # type: list
            add_invent_commands.extend(add_tmc_alcocodes)
        return add_invent_commands

    def get_tnvd(self):
        if len(self.tnvd_map) > 0:
            return self.tnvd_map
        for (root, dirs, files) in os.walk(self.work_dir, topdown=True):
            for f in list(filter(lambda x: x.endswith('.xml'), files)):
                filepath = os.path.normpath(root + "/" + f)
                tree = etree_lxml.parse(filepath)
                xml_root_node = tree.getroot()
                if xml_root_node.tag == 'CodeTNVED':
                    groups = xml_root_node.findall('group')
                    for group in groups:
                        self.tnvd_map[group.findtext('id')] = str(group.findtext('code')).replace("\n", "").replace(" ",
                                                                                                                    "")
        return self.tnvd_map

    @check_shop_code
    @get_aif_objects
    def parse_price(self, tree):
        self.log.debug("Загрузка справочника prices")
        prices_list = list()
        for action, elem in etree_lxml.iterwalk(tree, tag="item"):
            article = elem.attrib['article']
            prices = list(elem.findall('price'))
            for pr in prices:
                price = {}
                mrc = pr.find('MRC')
                value = pr.find('value')
                minprice = pr.find('minprice')

                price[value.tag] = value.text
                price[minprice.tag] = minprice.text
                price['article'] = article
                max_mrc = max(list(map(lambda y: y['MRCValue'], [{mrc.tag: mrc.text} for mrc in mrc.getchildren()]))) \
                    if (mrc is not None) else 0
                price['mrc'] = max_mrc
                prices_list.append(price)

        return prices_list

    @check_shop_code
    @get_aif_objects
    def parse_tprice(self, tree):
        """
        парсинг сущности tprice.
        пока что игнорируем МРЦ и minprice из xml - в нашей талице таких полей нет
        """
        self.log.debug("Загрузка справочника tprices")
        prices_list = list()
        # find anywhere in the tree
        price_type_code = tree.findtext(".//priceTypeCode")
        price_type_name = tree.findtext(".//priceTypeName")
        for action, elem in etree_lxml.iterwalk(tree, tag="item"):
            article = elem.attrib['article']
            prices = list(elem.findall('price'))
            for pr in prices:
                price_value = pr.findtext('value')
                """
                берем максимальную МРЦ из списка
                в качестве итоговой цены берем максимум между value из товара и МРЦ
                если нет МРЦ сравниваем value с minprice
                """
                mrc = pr.find('MRC')
                max_mrc = max(list(map(lambda y: y['MRCValue'], [{mrc.tag: mrc.text} for mrc in mrc.getchildren()]))) \
                    if mrc is not None else pr.findtext('minprice')
                price = {'value': max(float(price_value), float(max_mrc)), 'article': article, 'code': price_type_code,
                         'name': price_type_name}
                prices_list.append(price)

        return prices_list

    @get_aif_objects
    def parse_group(self, tree):
        groups = list()
        for action, elem in etree_lxml.iterwalk(tree, tag="group"):
            group = {g.tag: g.text for g in elem.getchildren()}
            groups.append(group)
        return groups

    @get_aif_objects
    def parse_kit(self, tree):
        kits = list()
        for action, elem in etree_lxml.iterwalk(tree, tag="item"):
            kit_items = list()
            article = elem.attrib["article"]
            for sub_item in elem.findall("subItem"):
                item = {g.tag: g.text for g in sub_item.getchildren()}
                item['parentcode'] = article
                kit_items.append(item)
            kits.append({'article': article, 'subItem': kit_items})
        return kits

    @get_aif_objects
    def parse_tax(self, tree):
        taxes = list()
        for action, elem in etree_lxml.iterwalk(tree, tag="group"):
            code = elem.findtext('id')
            for tax in elem.findall("tax"):
                tax_dict = dict()
                for t in list(tax.getchildren()):
                    tax_dict[t.tag] = t.text
                tax_dict["id"] = code
                taxes.append(tax_dict)
        return taxes


def invalidObjectsCount():
    return errorObjectsCount


def general_method(workDirectory, data, node):
    converter = SmagUkmXMLConverter(node)
    converter.work_dir = workDirectory
    return converter.parse(data)
